   /*
Try to make an external VFO fot the TS-520S.
Display 000Khz, VFO = 5.5Mhz
Display 600Khz, VFO = 4.9Hhz

Code for LCD not finished as yet so use an OLED display.

Some stuff is not as yet implemented but the basic VFO is working.
20240419 Have a play with band limiting and frequency  display?


20240428 put in band sense. HET osc / 65536 into T2, pin 5
  Change encoder button to pin 4.
Added RIT, RIT enable and it looks to be working. Not tested on a radio as yet.
  
*/

#define Version "TS-520S_20240502"

// comment out line below for LCD
#define USE_OLED

// a handy way to swap the encoder direction without rewiring
//#define SwapEncoder

#include "si5351.h"
#include "Wire.h"
#include "FreqCount.h"

// synthesiser default settings on start up.
unsigned long frequency =  3630000;     // 4 bytes required for frequency
unsigned long MaxFrq    =  5500000;
unsigned long MinFrq    =  4900000;
unsigned long DisplayFrq = 3630000;     // 4 bytes required for frequency
unsigned long step_size =     1000;     // default tuning increment (Hz)
         long RIT_freq  =        0;     // RIT offset for clockgen
unsigned long VFO       =  5000000;     // VFO frequency
unsigned long HETfreq   = 22895000;     // HET crystal osc frequency
unsigned long SecondIF  =  3393100;     //3395000
static signed char ham_band_index = 1;
static signed char old_band_index = 1;
unsigned long ham_band_count;           // het osc / 65536

unsigned long frequency0 =  1850000;          // 4 bytes required for frequency 
unsigned long frequency1 =  3630000;          // 4 bytes required for frequency
unsigned long frequency2 =  7100000;          // 4 bytes required for frequency
unsigned long frequency3 = 14200000;          // 4 bytes required for frequency
unsigned long frequency4 = 21250000;          // 4 bytes required for frequency
unsigned long frequency5 = 28250000;          // 4 bytes required for frequency
unsigned long frequency6 = 28750000;          // 4 bytes required for frequency
unsigned long frequency7 = 29400000;          // 4 bytes required for frequency
#define       RX                LOW
int           TRXmode         = RX;          // HIGH = TX, LOW = RX
#define       RIT_enable        LOW
int           oldRIT          =HIGH;

int32_t cal_factor = 33100; //  for my radio, found from Si5351 calibrator sketch.
byte updateFlag=HIGH;
int  temp = 0;              // temperary scratch pad
int loops = 0;
long encoder;                               // encoder direction: CCW=-1; NC=0; CW=1
Si5351 clockgen;                            // initalize clockgen

// port definitions
#define ENCODER_A  A0
#define ENCODER_B  A1
#define TRX        A2                      // 
#define ModeSelect A3                      // AM, USB, LSB 
// A4 = SDA
// A5 = SCL
#define RITpot     A6                  // RIT input
#define Buttons    A7                  // not used yet

#define TRX         2                  // PTT signal
#define RIT_On      3                  // Turn RIT in and off

#define ENCODER_BUTTON 4               // selects tuning decade
#define BandSense 5                    // read HETosc/65536 for band sense

#define MuteIn    6                    // 
#define Output1  13                    // not used yet
                           
#define ENCODER_INPUT_PORT PINC
int sideband = 0;   // 0 = AM, 1 = USB, 2 = LSB
int last_sideband = 0;
long RIT = 0;
long lastRIT = 0;

// Code for LCD not finished as yet so use an OLED display.
#ifdef USE_OLED
#include "oled_ui.h"
#else
#include "lcd_ui.h"
#endif

void setup() { 

  setup_display();

  // Start serial and initialize the Si5351
  Serial.begin(57600);
  clockgen.init(SI5351_CRYSTAL_LOAD_8PF, 0, 0);
  clockgen.set_correction(cal_factor, SI5351_PLL_INPUT_XO); // <<<<<<<this is found from Si5351 calibrator code.

  // configure rotary encoder & push-button
  pinMode(ENCODER_A, INPUT);     // encoder leads
  pinMode(ENCODER_B, INPUT);  
  pinMode(ENCODER_BUTTON, INPUT);
  pinMode(TRX, INPUT);                  // Low = TX
  pinMode(RIT_On, INPUT_PULLUP);        // 
  ReadBand();
  set_synth();
  display_freq_step();                  // display frequency & step-size
  FreqCount.begin(1000);
}

/******************************
 main loop
*******************************/
void loop() {
    if (FreqCount.available()) 
    { ham_band_count = FreqCount.read();
      ReadBand();
    if(ham_band_index != old_band_index)  // has the band changed?
      { old_band_index = ham_band_index;
        unsigned long ham_band[] = {frequency0,frequency1,frequency2,frequency3,frequency4,frequency5,frequency6,frequency7};
        frequency = ham_band[ham_band_index];     // change frequency band  
//        set_synth();
        updateFlag=HIGH;
      }
    }
    
if (digitalRead(TRX) !=TRXmode)
{TRXmode=(digitalRead(TRX)); // save for next time
updateFlag=HIGH;
}

if (digitalRead(RIT_On) !=oldRIT)     // has RIT state changed?
   { oldRIT = !oldRIT;
    if (oldRIT!=RIT_enable)          // turn RIT off
     { display.setCursor(50,0);       // TX, so clear the RIT display and set it to zero
       display.print("         ");
       display.display(); // Update screen
       RIT_freq = 0;
       set_synth();
     }
     else
     { lastRIT = 1000;                  // false reading to force an update
       DoRit();                         // update the RIT
     }
   }

loops ++;
  if (loops >50)                        // every 50th loop, check the RIT...
    { loops = 0;
    DoRit();  
    }
  
  check_step_size();              // check the frequency......
  encoder = read_encoder();                       // returns +1/0/-1 (cw/nc/ccw)
  if (encoder != 0) {
   frequency = frequency + (encoder * step_size); 
//   set_synth();
   updateFlag=HIGH;
  } 
 
  if(updateFlag==HIGH)
   { updateFlag = LOW;
     set_synth();
     display_freq_step();                       // display frequency & step-size
  }
}

/******************************
 read encoder direction
*******************************/
char read_encoder() {
  // definitions
  #define DELAY 100                               // number of polling-loops   
  static unsigned short integrator;               // encoder debounce
 
  /* the array values are determined by combining the previous and current A1,A0 
  Gray Code bit-patterns to form an array-index. +1 is assigned to all clockwise 
  (CW) indexes; -1 is assigned to all counter-clockwise (CCW) indexes; and 0 is 
  assigned to the remaining positions to indicate no movement*/   
  static char encoder[] = {0,-1,1,0,1,0,0,-1,-1,0,0,1,0,1,-1,0}; // SWAPPED PHASES
  //   static char encoder[] = {0,1,-1,0,-1,0,0,1,1,0,0,-1,0,-1,1,0};
  static unsigned char encoder_index = 0;
  
  static long count = 0;                         // CW rotation = +1; CCW rotation = -1
  static long this_reading;
  static long last_reading;
  char value;                                    // return value

  /* check for encoder rotation. The absolute "count" value changes as the encoder rotates.
  Typically there are four states (counts) per click, but often more due to contact bounce.
  Bounce doesn't matter so long as the "count" decreases" for CCW rotation; remains constant 
  when there is no rotation; and increases with CW rotation. Bounce can be completely 
  eliminated if we look for the "detent" code and integrate as shown below. */  
  encoder_index <<= 2;                           // shift previous bit-pattern left two places
  encoder_index |= (ENCODER_INPUT_PORT & 0x03);  // get current bit-pattern and combine with previous
  count += encoder[( encoder_index & 0x0f )];     
  
  // debounce encoder
  if ((ENCODER_INPUT_PORT & 0x03) == 3) {        // encoder on an indent (3 decimal == 00000011 binary)
    if (integrator < DELAY) {                    
      integrator++;
    } 
  } else {                                       // encoder not resting on indent
     if (integrator > 0) {
       integrator--;
     }
  }
  
  // prepare the "return" value
  if (integrator >= DELAY) {                     // encoder deemed to be stationary
    this_reading = count;

#ifdef SwapEncoder
    if (this_reading < last_reading) value =  1;
    if (this_reading == last_reading) value = 0;
    if (this_reading > last_reading) value = -1;
#else
    if (this_reading < last_reading) value = -1;
    if (this_reading == last_reading) value = 0;
    if (this_reading > last_reading) value =  1;
#endif
   
    last_reading = this_reading;                 // both readings now hold current "count"
    integrator = 0;                              // reset integrator
  } else {
    value = 0;
  }
  return(value);
}
  



/******************************
 check the step size
*******************************/
void check_step_size() {
  // delay definitions
  #define DELAY 30                             // DELAY = debounce(mS)*loops/second
  #define LONG_PUSH 50                        // number of loops considered "long"
  
  // variables
  static boolean transition = false;             // valid during button push
  static unsigned short integrator;              // varies from 0 to MAXIMUM
  static unsigned short push_duration;           // determines step size/direction
  
  // look for first high-to-low transition
  if (!transition && (digitalRead(ENCODER_BUTTON) == LOW)) {
    transition = true;
    integrator = 0;
    push_duration = 0;
  }
  
  // look for last low-to-high transition
  if (transition) {
    if (push_duration < (LONG_PUSH*2)) {         // track push duration
      push_duration++;
    }
    if (digitalRead(ENCODER_BUTTON) == HIGH) {   // ENCODER_BUTTON is HIGH
      if (integrator < DELAY) {        
        integrator++;
      } else {                                   // ENCODER_BUTTON is LOW
        if (integrator > 0) {
    integrator--;
  }
      }
    }
   
    // update the step_size
    if (integrator >= DELAY) { 
      transition = false;
      if (push_duration > LONG_PUSH) {          // LONG PUSH: decrease step size
        step_size = step_size/10;
        if (step_size < 10) {                    //
          step_size = 100000;                   // wrap around    
        }        
      } else {                                  // SHORT PUSH: increase step size
        step_size = step_size*10;
        if (step_size > 100000) {
          step_size = 10;                       // wrap around 
        }         
      }    
      display_freq_step();                      // update step-size     
    }
  }
}


/*******************************
set the Si5351 CLK0 frequency
 ******************************/
void set_synth ()
{
    switch (ham_band_index) {       // limited to SSB part of bands. 

    case 0:       //  1800000, 1875000
//      MinFrq   =  5500000;  
//      MaxFrq   =  5425000;
  frequency = constrain(frequency, 1840000, 1875000);
  frequency0 = frequency;
      HETfreq  = 10695000;
  break;
  
    case 1:       //  3525000, 3700000 
//      MinFrq   =  5500000;  
//      MaxFrq   =  5200000;
  frequency = constrain(frequency, 3525000, 3700000);
  frequency1 = frequency;
      HETfreq  = 12395000;
  break;

    case 2:       //  7040000, 7300000 
//      MinFrq   =  5500000;  
//      MaxFrq   =  5200000;
  frequency = constrain(frequency, 7040000, 7300000);
  frequency2 = frequency;
      HETfreq  = 15895000;
  break;

    case 3:       // 14112000, 14350000
//      MinFrq   =  5400000;  
//      MaxFrq   =  5150000;
  frequency = constrain(frequency, 14112000, 14350000);
  frequency3 = frequency;
      HETfreq  = 22895000;
  break;

    case 4:       // 21000000, 21450000
//      MinFrq   =  5500000;  
//      MaxFrq   =  5050000;
  frequency = constrain(frequency, 21150000, 21450000);
  frequency4 = frequency;
      HETfreq  = 29895000;
  break;

    case 5:       // 28000000, 28600000
//      MinFrq   =  5500000;  
//      MaxFrq   =  4900000;
  frequency = constrain(frequency, 28000000, 28600000);
  frequency5 = frequency;      
      HETfreq  = 36895000;
  break;

    case 6:       // 28500000, 28900000
//      MinFrq   =  5500000;  
//      MaxFrq   =  4900000;
  frequency = constrain(frequency, 28500000, 28900000);
  frequency6 = frequency;
      HETfreq  = 37395000;
  break;

    case 7:       // 29100000, 29700000
//      MinFrq   =  5500000;  
//      MaxFrq   =  5490000;
  frequency = constrain(frequency, 29100000, 29700000);
  frequency7 = frequency;
      HETfreq  = 37995000;
  break;

}

{ VFO = (HETfreq -frequency -RIT_freq -SecondIF );
  clockgen.set_freq((VFO*100), SI5351_CLK0); // "VFO"
}
}

void ReadBand()
{ if (ham_band_count > 575)   // 580
    { ham_band_index = 7;
    }
  else
  if (ham_band_count > 567)   // 571
    { ham_band_index = 6;
    }
  else
  if (ham_band_count > 500)   // 563
    { ham_band_index = 5;
    }
  else
  if (ham_band_count > 400)   // 456
    { ham_band_index = 4;
    }
  else 
  if (ham_band_count > 300)   // 349  
    { ham_band_index = 3;
    }
  else
  if (ham_band_count > 200)   //243
    { ham_band_index = 2;
    }
  else
  if (ham_band_count > 180)   // 189
    { ham_band_index = 1;
    }
  else
    { ham_band_index = 0;     //163
    }    
}

void DoRit()
{ if (TRXmode==RX && (digitalRead(RIT_On)==LOW)) // check the RIT?
   { RIT = (analogRead(RITpot));        // try to filter the noise out....
     if ( RIT==(analogRead(RITpot)))     // read twice the same in a row...
      { if ( RIT==(analogRead(RITpot)))   // read thrice the same in a row...
         { if (RIT!=lastRIT)
            { lastRIT = RIT;         // save history
              display.setCursor(50,0);
              display.print("RIT=     ");
              RIT_freq = (RIT-512);
              display.setCursor(80,0);
              if (RIT_freq >=0)      
               {display.print(" ");  // space over -ve symbol place if +ve        
               }
              display.print(RIT_freq);          
              display.display(); // Update screen
              set_synth();
            }           
          }
       }
    }
 
}

/* HF Ham bands
Band        UK Allocation          USA Allocation    Australia Allocation
(metres)          Mhz                   Mhz                 Mhz
 160         1.810 -  2.000        1.800 -  2.000        1.800 -  1.875
  80         3.500 -  3.800        3.500 -  4.000        3.500 -  3.800
  40         7.000 -  7.200        7.000 -  7.300        7.000 -  7.300
  30        10.100 - 10.150       10.100 - 10.150       10.100 - 10.150
  20        14.100 - 14.350       14.100 - 14.350       14.100 - 14.350
  17        18.068 - 18.168       18.068 - 18.168       18.068 - 18.168
  15        21.000 - 21.450       21.000 - 21.450       21.000 - 21.450 
  12        24.890 - 24.990       24.890 - 24.990       24.890 - 24.990
  10        28.000 - 29.700       28.000 - 29.700       28.000 - 29.700 
 */
